@@ -29,14 +29,14 @@
 
 segm3D <- function(y,weighted=TRUE,
                    sigma2=NULL,mask=NULL,hinit=NULL,hmax=NULL,
-                   ladjust=1,graph=FALSE,wghts=NULL,
-                   df=100,h0=c(0,0,0),res=NULL, resscale=NULL,
-                   ddim=NULL,delta=0,alpha=.05,restricted=FALSE) {
+                   ladjust=1,wghts=NULL,
+                   df=100,h0=c(0,0,0),residuals=NULL, resscale=NULL,
+                   delta=0,alpha=.05,restricted=FALSE) {
 #
 #
 #  Auxilary functions
       getkrval <- function(df,ladj,fov,k,alpha){
-#  define threshold for segmantation
+#  define threshold for segmentation
       dfinv <- 1/df
       lfov <- log(fov)
       idffov <- dfinv*log(fov)
@@ -52,19 +52,16 @@ segm3D <- function(y,weighted=TRUE,
 # first check arguments and initialize
 #
    args <- match.call()
-   if(is.null(ddim)) ddim <- dim(y)
-# test dimension of data (vector of 3D) and define dimension related stuff
-   d <- 3
-   dy <- dim(y)
-   n1 <- dy[1]
-   n2 <- dy[2]
-   n3 <- dy[3]
-   n <- n1*n2*n3
-   nt <- ddim[4]
-   if (length(dy)==d+1) {
-      dim(y) <- dy[1:3]
-   } else if (length(dy)!=d) {
-      stop("y has to be 3 dimensional")
+   dmask <- dim(mask)
+   n1 <- dmask[1]
+   n2 <- dmask[2]
+   n3 <- dmask[3]
+   nvoxel <- sum(mask)
+   nt <- dim(residuals)[1]
+   position <- array(0,dmask)
+   position[mask] <- 1:nvoxel
+   if (length(y)!=nvoxel) {
+     stop("segm3D: y needs to have length sum(mask)")
    }
 # set the code for the kernel (used in lkern) and set lambda
    lkern <- 1
@@ -77,24 +74,13 @@ segm3D <- function(y,weighted=TRUE,
    if (is.null(hmax)) hmax <- 5    # uses a maximum of about 520 points
 # estimate variance in the gaussian case if necessary
 # deal with homoskedastic Gaussian case by extending sigma2
-   if (length(sigma2)==1) sigma2<-array(sigma2,dy[1:3])
-   if (length(sigma2)!=n) stop("sigma2 does not have length 1 or same length as y")
-   dim(sigma2) <- dy[1:3]
-   if(is.null(mask)) mask <- array(TRUE,dy[1:3])
-   mask[sigma2>=1e16] <- FALSE
+   if (length(sigma2)==1) sigma2<-array(sigma2,nvoxel)
+   if (length(sigma2)!=nvoxel) stop("sigma2 does not have length 1 or same length as y")
 #  in these points sigma2 probably contains NA's
    sigma2 <- 1/sigma2 #  taking the invers yields simpler formulaes
 # deal with homoskedastic Gaussian case by extending sigma2
-   residuals <- readBin(res,"integer",prod(ddim),2)
   cat("\nfmri.smooth: first variance estimate","\n")
-  varest0 <- .Fortran(C_ivar,as.double(residuals),
-                           as.double(resscale),
-                           as.integer(mask),
-                           as.integer(n1),
-                           as.integer(n2),
-                           as.integer(n3),
-                           as.integer(nt),
-                           var = double(n1*n2*n3))$var
+  varest0 <- aws::residualVariance(residuals, mask, resscale=1, compact=TRUE)
    vq <- varest0*sigma2
    if (is.null(wghts)) wghts <- c(1,1,1)
    hinit <- hinit/wghts[1]
@@ -102,9 +88,9 @@ segm3D <- function(y,weighted=TRUE,
    wghts <- wghts[2:3]/wghts[1]
    tobj <- list(bi= sigma2)
    theta <- y
-   segm <- array(0,dy[1:3])
+   segm <- numeric(nvoxel)
    varest <- varest0
-   maxvol <- getvofh(hmax,lkern,wghts)
+   maxvol <- aws::getvofh(hmax,lkern,wghts)
    fov <- sum(mask)
    kstar <- as.integer(log(maxvol)/log(1.25))
    steps <- kstar+1
@@ -120,23 +106,21 @@ segm3D <- function(y,weighted=TRUE,
 #  just to ensure monotonicity of thresh with kmax, there exist a few parameter configurations
 #  where the approximation formula does not ensure monotonicity
    cat("FOV",fov,"delta",delta,"thresh",thresh,"ladjust",ladjust,"lambda",lambda,"df",df,"\n")
-   residuals <- residuals*resscale
-   kv <- array(0,dy[1:3])
 #
 #   need these values to compute variances
 #
    while (k<=kstar) {
-      hakt0 <- gethani(1,10,lkern,1.25^(k-1),wghts,1e-4)
-      hakt <- gethani(1,10,lkern,1.25^k,wghts,1e-4)
+      hakt0 <- aws::gethani(1,10,lkern,1.25^(k-1),wghts,1e-4)
+      hakt <- aws::gethani(1,10,lkern,1.25^k,wghts,1e-4)
       cat("step",k,"bandwidth",signif(hakt,3)," ")
-      dlw <- (2*trunc(hakt/c(1,wghts))+1)[1:d]
+      dlw <- (2*trunc(hakt/c(1,wghts))+1)[1:3]
       hakt0 <- hakt
       bi0 <- tobj$bi
       tobj <- .Fortran(C_segm3d,
                        as.double(y),
                        as.double(residuals),
                        as.double(sigma2),
-                       as.integer(!mask),
+                       as.integer(position),
                        as.integer(weighted),
                        as.integer(n1),
                        as.integer(n2),
@@ -147,11 +131,11 @@ segm3D <- function(y,weighted=TRUE,
                        as.double(lambda0),
                        as.double(theta),
                        bi=as.double(bi0),
-                       thnew=double(n1*n2*n3),
+                       thnew=double(nvoxel),
                        double(prod(dlw)),
                        as.double(wghts),
                        double(nt),#swres
-                       double(n1*n2*n3),#pvalues
+                       double(nvoxel),#pvalues
                        segm=as.integer(segm),
                        as.double(delta),
                        as.double(thresh),
@@ -161,21 +145,9 @@ segm3D <- function(y,weighted=TRUE,
                        varest=as.double(varest),
                        as.integer(restricted))[c("bi","thnew","hakt","segm","varest")]
       gc()
-      theta <- array(tobj$thnew,dy[1:3])
-      segm <- array(tobj$segm,dy[1:3])
-      varest <- array(tobj$varest,dy[1:3])
-      dim(tobj$bi) <- dy[1:3]
-      if (graph) {
-         par(mfrow=c(2,2),mar=c(1,1,3,.25),mgp=c(2,1,0))
-         image(y[,,n3%/%2+1],col=gray((0:255)/255),xaxt="n",yaxt="n")
-         title(paste("Observed Image  min=",signif(min(y),3)," max=",signif(max(y),3)))
-         image(theta[,,n3%/%2+1],col=gray((0:255)/255),xaxt="n",yaxt="n")
-         title(paste("Reconstruction  h=",signif(hakt,3)," min=",signif(min(theta),3),"   max=",signif(max(theta),3)))
-         image(segm[,,n3%/%2+1]>0,col=gray((0:255)/255),xaxt="n",yaxt="n")
-         title(paste("Segmentation  h=",signif(hakt,3)," detected=",sum(segm>0)))
-         image(tobj$bi[,,n3%/%2+1],col=gray((0:255)/255),xaxt="n",yaxt="n")
-         title(paste("Sum of weights: min=",signif(min(tobj$bi),3)," mean=",signif(mean(tobj$bi),3)," max=",signif(max(tobj$bi),3)))
-      }
+      theta <- tobj$thnew
+      segm <- tobj$segm
+      varest <- tobj$varest
       if (max(total) >0) {
          cat(signif(total[k],2)*100,"%                 \r",sep="")
       }
